### Author: Dag Wieers <dag@wieers.com>

### FIXME: Should read /var/log/mail/statistics or /etc/mail/statistics (format ?)

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'sendmail'
        self.vars = ('queue',)
        self.type = 'd'
        self.width = 4
        self.scale = 100

    def check(self):
        if not os.access('/var/spool/mqueue', os.R_OK):
            raise Exception('Cannot access sendmail queue')

    def extract(self):
        self.val['queue'] = len(glob.glob('/var/spool/mqueue/qf*'))

# vim:ts=4:sw=4:et



### Author: <gianfranco@mongodb.com>

global mongodb_user
mongodb_user = os.getenv('DSTAT_MONGODB_USER') or os.getenv('USER')

global mongodb_pwd
mongodb_pwd = os.getenv('DSTAT_MONGODB_PWD')

global mongodb_host
mongodb_host = os.getenv('DSTAT_MONGODB_HOST') or '127.0.0.1:27017'

class dstat_plugin(dstat):
  """
  Plugin for MongoDB.
  """
  def __init__(self):
    global pymongo
    import pymongo

    try:
      self.m = pymongo.MongoClient(mongodb_host)
      if mongodb_pwd:
        self.m.admin.authenticate(mongodb_user, mongodb_pwd)
      self.db = self.m.admin
    except Exception as e:
      raise Exception('Cannot interface with MongoDB server: %s' % e)

    line = self.db.command("serverStatus")
    if 'storageEngine' in line:
      self.storageEngine = line.get('storageEngine').get('name')
    else:
      self.storageEngine = 'mmapv1'

    self.name    = 'mongodb mem'
    self.nick    = ('res', 'virt')
    self.vars    = ('mem.resident', 'mem.virtual')
    self.type    = 'd'
    self.width   = 5
    self.scale   = 2
    self.lastVal = {}

    if self.storageEngine == 'mmapv1':
      self.nick = self.nick + ('map', 'mapj', 'flt')
      self.vars = self.vars + ('mem.mapped', 'mem.mappedWithJournal', 'extra_info.page_faults')


  def extract(self):
    status = self.db.command("serverStatus")

    for name in self.vars:
      if name in ('extra_info.page_faults'):
        if not name in self.lastVal:
          self.lastVal[name] = int(self.getDoc(status, name))
        self.val[name] = (int(self.getDoc(status, name)) - self.lastVal[name])
        self.lastVal[name] = self.getDoc(status, name)
      else:
        self.val[name] = (int(self.getDoc(status, name)))



  def getDoc(self, dic, doc):
    par = doc.split('.')
    sdic = dic
    for p in par:
      sdic = sdic.get(p)

    return sdic



### Author: Dag Wieers <dag@wieers.com>

### Dstat most expensive process plugin
### Displays the name of the most expensive process

### More information:
###    http://lwn.net/Articles/317814/

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'out of memory'
        self.vars = ('kill score',)
        self.type = 's'
        self.width = 18
        self.scale = 0

    def check(self):
        if not os.access('/proc/self/oom_score', os.R_OK):
            raise Exception('Kernel does not support /proc/pid/oom_score, use at least 2.6.11.')

    def extract(self):
        self.output = ''
        self.val['max'] = 0.0
        for pid in proc_pidlist():
            try:
                ### Extract name
                name = proc_splitline('/proc/%s/stat' % pid)[1][1:-1]

                ### Using dopen() will cause too many open files
                l = proc_splitline('/proc/%s/oom_score' % pid)
            except IOError:
                continue
            except IndexError:
                continue

            if len(l) < 1: continue
            oom_score = int(l[0])

            ### Is it a new topper ?
            if oom_score <= self.val['max']: continue

            self.val['max'] = oom_score
            self.val['name'] = getnamebypid(pid, name)
            self.val['pid'] = pid

        if self.val['max'] != 0.0:
            self.output = '%-*s%s' % (self.width-4, self.val['name'][0:self.width-4], cprint(self.val['max'], 'f', 4, 1000))

        ### Debug (show PID)
#       self.output = '%*s %-*s' % (5, self.val['pid'], self.width-6, self.val['name'])

    def showcsv(self):
        return '%s / %d%%' % (self.val['name'], self.val['max'])

# vim:ts=4:sw=4:et



### Dstat most expensive process plugin
### Displays the name of the most expensive process
###
### Authority: dag@wieers.com

global cpunr

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'most waiting for'
        self.vars = ('child process',)
        self.type = 's'
        self.width = 16
        self.scale = 0

    def extract(self):
        self.set2 = {}
        self.val['max'] = 0.0
        for pid in proc_pidlist():
            try:
                ### Using dopen() will cause too many open files
                l = proc_splitline('/proc/%s/stat' % pid)
            except IOError:
                continue

            if len(l) < 15: continue

            ### Reset previous value if it doesn't exist
            if pid not in self.set1:
                self.set1[pid] = 0

            self.set2[pid] = int(l[15]) + int(l[16])
            usage = (self.set2[pid] - self.set1[pid]) * 1.0 / elapsed / cpunr

            ### Is it a new topper ?
            if usage <= self.val['max']: continue

            self.val['max'] = usage
            self.val['name'] = getnamebypid(pid, l[1][1:-1])
            self.val['pid'] = pid

        ### Debug (show PID)
#       self.val['process'] = '%*s %-*s' % (5, self.val['pid'], self.width-6, self.val['name'])

        if step == op.delay:
            self.set1 = self.set2

    def show(self):
        if self.val['max'] == 0.0:
            return '%-*s' % (self.width, '')
        else:
            return '%s%-*s%s' % (theme['default'], self.width-3, self.val['name'][0:self.width-3], cprint(self.val['max'], 'p', 3, 34))

    def showcsv(self):
        return '%s / %d%%' % (self.val['name'], self.val['max'])

# vim:ts=4:sw=4:et



### Author: HIROSE Masaaki <hirose31 _at_ gmail.com>

global mysql_options
mysql_options = os.getenv('DSTAT_MYSQL') or ''

global target_status
global _basic_status
global _extra_status
_basic_status = (
    ('Queries'                       , 'qps'),
    ('Com_select'                    , 'sel/s'),
    ('Com_insert'                    , 'ins/s'),
    ('Com_update'                    , 'upd/s'),
    ('Com_delete'                    , 'del/s'),

    ('Connections'                   , 'con/s'),
    ('Threads_connected'             , 'thcon'),
    ('Threads_running'               , 'thrun'),
    ('Slow_queries'                  , 'slow'),
    )
_extra_status = (
    ('Innodb_rows_read'              , 'r#read'),
    ('Innodb_rows_inserted'          , 'r#ins'),
    ('Innodb_rows_updated'           , 'r#upd'),
    ('Innodb_rows_deleted'           , 'r#del'),

    ('Innodb_data_reads'               , 'rdphy'),
    ('Innodb_buffer_pool_read_requests', 'rdlgc'),
    ('Innodb_data_writes'              , 'wrdat'),
    ('Innodb_log_writes'               , 'wrlog'),

    ('innodb_buffer_pool_pages_dirty_pct', '%dirty'),
    )

global calculating_status
calculating_status = (
    'Innodb_buffer_pool_pages_total',
    'Innodb_buffer_pool_pages_dirty',
    )

global gauge
gauge = {
    'Slow_queries'                    : 1,
    'Threads_connected'               : 1,
    'Threads_running'                 : 1,
    }

class dstat_plugin(dstat):
    """
    mysql5-innodb, mysql5-innodb-basic, mysql5-innodb-extra

    display various metircs on MySQL5 and InnoDB.
    """
    def __init__(self):
        self.name = 'MySQL5 InnoDB '
        self.type = 'd'
        self.width = 5
        self.scale = 1000

    def check(self):
        if self.filename.find("basic") >= 0:
            target_status = _basic_status
            self.name += 'basic'
        elif self.filename.find("extra") >= 0:
            target_status = _extra_status
            self.name += 'extra'
        elif self.filename.find("full") >= 0:
            target_status = _basic_status + _extra_status
            self.name += 'full'
        else:
            target_status = _basic_status + _extra_status
            self.name += 'full'

        self.vars = tuple( map((lambda e: e[0]), target_status) )
        self.nick = tuple( map((lambda e: e[1]), target_status) )

        mysql_candidate = ('/usr/bin/mysql', '/usr/local/bin/mysql')
        mysql_cmd = ''
        for mc in mysql_candidate:
            if os.access(mc, os.X_OK):
                mysql_cmd = mc
                break

        if mysql_cmd:
            try:
                self.stdin, self.stdout, self.stderr = dpopen('%s -n %s' % (mysql_cmd, mysql_options))
            except IOError:
                raise Exception('Cannot interface with MySQL binary')
            return True
        raise Exception('Needs MySQL binary')

    def extract(self):
        try:
            self.stdin.write('show global status;\n')
            for line in readpipe(self.stdout):
                if line == '':
                    break
                s = line.split()
                if s[0] in self.vars:
                    self.set2[s[0]] = float(s[1])
                elif s[0] in calculating_status:
                    self.set2[s[0]] = float(s[1])

            for k in self.vars:
                if k in gauge:
                    self.val[k] = self.set2[k]
                elif k == 'innodb_buffer_pool_pages_dirty_pct':
                    self.val[k] = self.set2['Innodb_buffer_pool_pages_dirty'] / self.set2['Innodb_buffer_pool_pages_total'] * 100
                else:
                    self.val[k] = (self.set2[k] - self.set1[k]) * 1.0 / elapsed

            if step == op.delay:
                self.set1.update(self.set2)

        except IOError as e:
            if op.debug > 1: print('%s: lost pipe to mysql, %s' % (self.filename, e))
            for name in self.vars: self.val[name] = -1

        except Exception as e:
            if op.debug > 1: print('%s: exception' % (self.filename, e))
            for name in self.vars: self.val[name] = -1




### Author: Dag Wieers <dag$wieers,com>

global mysql_options
mysql_options = os.getenv('DSTAT_MYSQL')

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'innodb ops'
        self.nick = ('ins', 'upd', 'del', 'rea')
        self.vars = ('inserted', 'updated', 'deleted', 'read')
        self.type = 'f'
        self.width = 3
        self.scale = 1000

    def check(self):
        if os.access('/usr/bin/mysql', os.X_OK):
            try:
                self.stdin, self.stdout, self.stderr = dpopen('/usr/bin/mysql -n %s' % mysql_options)
            except IOError:
                raise Exception('Cannot interface with MySQL binary')
            return True
        raise Exception('Needs MySQL binary')

    def extract(self):
        try:
            self.stdin.write('show engine innodb status\G\n')
            line = greppipe(self.stdout, 'Number of rows inserted')

            if line:
                l = line.split()
                self.set2['inserted'] = int(l[4].rstrip(','))
                self.set2['updated'] = int(l[6].rstrip(','))
                self.set2['deleted'] = int(l[8].rstrip(','))
                self.set2['read'] = int(l[10])

            for name in self.vars:
                self.val[name] = (self.set2[name] - self.set1[name]) * 1.0 / elapsed

            if step == op.delay:
                self.set1.update(self.set2)

        except IOError as e:
            if op.debug > 1: print('%s: lost pipe to mysql, %s' % (self.filename, e))
            for name in self.vars: self.val[name] = -1

        except Exception as e:
            if op.debug > 1: print('%s: exception' % (self.filename, e))
            for name in self.vars: self.val[name] = -1

# vim:ts=4:sw=4:et



### Dstat all I/O process plugin
### Displays all processes' I/O read/write stats and CPU usage
###
### Authority: Guillermo Cantu Luna

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'most expensive cpu process'
        self.vars = ('process              pid  cpu read write',)
        self.type = 's'
        self.width = 40
        self.scale = 0
        self.pidset1 = {}

    def check(self):
        if not os.access('/proc/self/io', os.R_OK):
            raise Exception('Kernel has no per-process I/O accounting [CONFIG_TASK_IO_ACCOUNTING], use at least 2.6.20')
        return True

    def extract(self):
        self.output = ''
        self.pidset2 = {}
        self.val['cpu_usage'] = 0
        for pid in proc_pidlist():
            try:
                ### Reset values
                if pid not in self.pidset2:
                    self.pidset2[pid] = {'rchar:': 0, 'wchar:': 0, 'cputime:': 0, 'cpuper:': 0}
                if pid not in self.pidset1:
                    self.pidset1[pid] = {'rchar:': 0, 'wchar:': 0, 'cputime:': 0, 'cpuper:': 0}

                ### Extract name
                name = proc_splitline('/proc/%s/stat' % pid)[1][1:-1]

                ### Extract counters
                for l in proc_splitlines('/proc/%s/io' % pid):
                    if len(l) != 2: continue
                    self.pidset2[pid][l[0]] = int(l[1])

                ### Get CPU usage
                l = proc_splitline('/proc/%s/stat' % pid)
                if len(l) < 15:
                    cpu_usage = 0.0
                else:
                    self.pidset2[pid]['cputime:'] = int(l[13]) + int(l[14])
                    cpu_usage = (self.pidset2[pid]['cputime:'] - self.pidset1[pid]['cputime:']) * 1.0 / elapsed / cpunr

            except ValueError:
                continue
            except IOError:
                continue
            except IndexError:
                continue

            read_usage = (self.pidset2[pid]['rchar:'] - self.pidset1[pid]['rchar:']) * 1.0 / elapsed
            write_usage = (self.pidset2[pid]['wchar:'] - self.pidset1[pid]['wchar:']) * 1.0 / elapsed

            ### Get the process that spends the most jiffies
            if cpu_usage > self.val['cpu_usage']:
                self.val['read_usage'] = read_usage
                self.val['write_usage'] = write_usage
                self.val['pid'] = pid
                self.val['name'] = getnamebypid(pid, name)
                self.val['cpu_usage'] = cpu_usage

        if step == op.delay:
            self.pidset1 = self.pidset2

        if self.val['cpu_usage'] != 0.0:
            self.output = '%-*s%s%-5s%s%s%%%s%s' % (self.width-14-len(pid), self.val['name'][0:self.width-14-len(pid)], color['darkblue'], self.val['pid'], cprint(self.val['cpu_usage'], 'f', 3, 34), color['darkgray'],cprint(self.val['read_usage'], 'd', 5, 1024), cprint(self.val['write_usage'], 'd', 5, 1024))


    def showcsv(self):
        return 'Top: %s\t%s\t%s\t%s' % (self.val['name'][0:self.width-20], self.val['cpu_usage'], self.val['read_usage'], self.val['write_usage'])



### Author: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'wifi'
        self.nick = ('lnk', 's/n')
        self.type = 'd'
        self.width = 3
        self.scale = 34
        self.cols = 2

    def check(self):
        global iwlibs
        from pythonwifi import iwlibs

    def vars(self):
        return iwlibs.getNICnames()

    def extract(self):
        for name in self.vars:
            wifi = iwlibs.Wireless(name)
            stat, qual, discard, missed_beacon = wifi.getStatistics()
#           print(qual.quality, qual.signallevel, qual.noiselevel)
            if qual.quality == 0 or qual.signallevel == -101 or qual.noiselevel == -101 or qual.signallevel == -256 or qual.noiselevel == -256:
                self.val[name] = ( -1, -1 )
            else:
                self.val[name] = ( qual.quality, qual.signallevel * 100 / qual.noiselevel )

# vim:ts=4:sw=4:et



### Authority: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    """
    Most expensive CPU process.

    Displays the process that uses the CPU the most during the monitored
    interval. The value displayed is the percentage of CPU time for the total
    amount of CPU processing power. Based on per process CPU information.
    """
    def __init__(self):
        self.name = 'most expensive'
        self.vars = ('cpu process',)
        self.type = 's'
        self.width = 16
        self.scale = 0
        self.pidset1 = {}

    def extract(self):
        self.output = ''
        self.pidset2 = {}
        self.val['max'] = 0.0
        for pid in proc_pidlist():
            try:
                ### Using dopen() will cause too many open files
                l = proc_splitline('/proc/%s/stat' % pid)
            except IOError:
                continue

            if len(l) < 15: continue

            ### Reset previous value if it doesn't exist
            if pid not in self.pidset1:
                self.pidset1[pid] = 0

            self.pidset2[pid] = int(l[13]) + int(l[14])
            usage = (self.pidset2[pid] - self.pidset1[pid]) * 1.0 / elapsed / cpunr

            ### Is it a new topper ?
            if usage < self.val['max']: continue

            name = l[1][1:-1]

            self.val['max'] = usage
            self.val['pid'] = pid
            self.val['name'] = getnamebypid(pid, name)
#            self.val['name'] = name

        if self.val['max'] != 0.0:
            self.output = '%-*s%s' % (self.width-3, self.val['name'][0:self.width-4], cprint(self.val['max'], 'f', 3, 34))

        ### Debug (show PID)
#        self.output = '%*s %-*s' % (5, self.val['pid'], self.width-6, self.val['name'])

        if step == op.delay:
            self.pidset1 = self.pidset2

    def showcsv(self):
        return '%s / %d%%' % (self.val['name'], self.val['max'])

# vim:ts=4:sw=4:et



# Author: Roberto Polli <rpolli@redhat.com>
#
# This plugin shows jvm stats using the JVM_PID environment variable.
# Requires the presence of the /tmp/hsperfdata_* directory and
#  files created when running java with the profiler enabled.
#


class dstat_plugin(dstat):

    def __init__(self):
        self.name = 'jvm mem ops '
        self.vars = ('fgc', 'heap', 'heap%', 'perm', 'perm%')
        self.type = 'f'
        self.width = 5
        self.scale = 1000

    def check(self):
        if not os.access('/usr/bin/jstat', os.X_OK):
            raise Exception('Needs jstat binary')
        try:
            self.jvm_pid = int(os.environ.get('JVM_PID', 0))
        except Exception:
            self.jvm_pid = 0

        return True

    @staticmethod
    def _to_float(s):
        return float(s.replace(",", "."))

    @staticmethod
    def _cmd_splitlines(cmd):
        for l in os.popen(cmd):
            yield l.strip().split()

    def extract(self):
        from collections import namedtuple
        try:
            lines = self._cmd_splitlines(
                '/usr/bin/jstat -gc %s' % self.jvm_pid)
            headers = next(lines)
            DStatParser = namedtuple('DStatParser', headers)
            line = next(lines)
            if line:
                stats = DStatParser(*[self._to_float(x) for x in line])
                # print(stats)
                self.set2['cls'] = 0
                self.set2['fgc'] = int(stats.FGC)
                self.set2['heap'] = (
                    stats.S0C + stats.S1C + stats.EC + stats.OC)
                self.set2['heapu'] = (
                    stats.S0U + stats.S1U + stats.EU + stats.OU)

                # Use MetaSpace on jdk8
                try:
                    self.set2['perm'] = stats.PC
                    self.set2['permu'] = stats.PU
                except AttributeError:
                    self.set2['perm'] = stats.MC
                    self.set2['permu'] = stats.MU

            # Evaluate statistics on memory usage.
            for name in ('heap', 'perm'):
                self.set2[name + '%'] = 100 * self.set2[
                    name + 'u'] / self.set2[name]
                self.set2[name] /= 1024

            for name in self.vars:
                self.val[name] = self.set2[name]

            if step == op.delay:
                self.set1.update(self.set2)

        except IOError as e:
            if op.debug > 1:
                print('%s: lost pipe to jstat, %s' % (self.filename, e))
            for name in self.vars:
                self.val[name] = -1

        except Exception as e:
            if op.debug > 1:
                print('%s: exception' % e)
            for name in self.vars:
                self.val[name] = -1

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'extended nfs3 server operations'
        self.nick = ('null', 'gatr', 'satr', 'look', 'aces', 'rdln', 'read', 'writ', 'crea', 'mkdr', 'syml', 'mknd', 'rm', 'rmdr', 'ren', 'link', 'rdir', 'rdr+', 'fstt', 'fsnf', 'path', 'cmmt')
        self.vars = ('null', 'getattr', 'setattr', 'lookup', 'access', 'readlink', 'read', 'write', 'create', 'mkdir', 'symlink', 'mknod', 'remove', 'rmdir', 'rename', 'link', 'readdir', 'readdirplus', 'fsstat', 'fsinfo', 'pathconf', 'commit')
        self.type = 'd'
        self.width = 5
        self.scale = 1000
        self.open('/proc/net/rpc/nfsd')

    def check(self):
        info(1, 'Module %s is still experimental.' % self.filename)

    def extract(self):
        for l in self.splitlines():
            if not l or l[0] != 'proc3': continue
            for i, name in enumerate(self.vars):
                self.set2[name] = int(l[i+2])

        for name in self.vars:
            self.val[name] = (self.set2[name] - self.set1[name]) * 1.0 / elapsed

        if step == op.delay:
            self.set1.update(self.set2)

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag$wieers,com>

global mysql_options
mysql_options = os.getenv('DSTAT_MYSQL')

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'innodb pool'
        self.nick = ('crt', 'rea', 'wri')
        self.vars = ('created', 'read', 'written')
        self.type = 'f'
        self.width = 3
        self.scale = 1000

    def check(self): 
        if not os.access('/usr/bin/mysql', os.X_OK):
            raise Exception('Needs MySQL binary')
        try:
            self.stdin, self.stdout, self.stderr = dpopen('/usr/bin/mysql -n %s' % mysql_options)
        except IOError as e:
            raise Exception('Cannot interface with MySQL binary (%s)' % e)

    def extract(self):
        try:
            self.stdin.write('show engine innodb status\G\n')
            line = greppipe(self.stdout, 'Pages read ')

            if line:
                l = line.split()
                self.set2['read'] = int(l[2].rstrip(','))
                self.set2['created'] = int(l[4].rstrip(','))
                self.set2['written'] = int(l[6])

            for name in self.vars:
                self.val[name] = (self.set2[name] - self.set1[name]) * 1.0 / elapsed

            if step == op.delay:
                self.set1.update(self.set2)

        except IOError as e:
            if op.debug > 1: print('%s: lost pipe to mysql, %s' % (self.filename, e))
            for name in self.vars: self.val[name] = -1

        except Exception as e:
            if op.debug > 1: print('%s: exception: %s' % (self.filename, e))
            for name in self.vars: self.val[name] = -1

# vim:ts=4:sw=4:et



### Author: Bert de Bruijn <bert+dstat$debruijn,be>

### VMware cpu stats
### Displays CPU stats coming from the hypervisor inside VMware VMs.
### The vmGuestLib API from VMware Tools needs to be installed

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'vm cpu'
        self.vars = ('used', 'stolen', 'elapsed')
        self.nick = ('usd', 'stl')
        self.type = 'p'
        self.width = 3
        self.scale = 100
        self.cpunr = getcpunr()

    def check(self):
        try:
            global vmguestlib
            import vmguestlib

            self.gl = vmguestlib.VMGuestLib()
        except:
            raise Exception('Needs python-vmguestlib module')

    def extract(self):
        self.gl.UpdateInfo()
        self.set2['elapsed'] = self.gl.GetElapsedMs()
        self.set2['stolen'] = self.gl.GetCpuStolenMs()
        self.set2['used'] = self.gl.GetCpuUsedMs()

        for name in ('stolen', 'used'):
            self.val[name] = (self.set2[name] - self.set1[name]) * 100 / (self.set2['elapsed'] - self.set1['elapsed']) / self.cpunr

        if step == op.delay:
            self.set1.update(self.set2)

# vim:ts=4:sw=4



### Author: Bert de Bruijn <bert+dstat@debruijn.be>

class dstat_plugin(dstat):
    """
    Recovery state of software RAID rebuild.

    Prints completed recovery percentage and rebuild speed of the md device
    that is actively being recovered or resynced.

    If no devices are being rebuilt, it displays 100%, 0B. If instead
    multiple devices are being rebuilt, it displays the total progress
    and total throughput.
    """

    def __init__(self):
        self.name = 'sw raid'
        self.type = 's'
        self.scale = 0
        self.nick = ('pct speed', )
        self.width = 9
        self.vars = ('text', )
        self.open('/proc/mdstat')

    def check(self):
        if not os.path.exists('/proc/mdstat'):
            raise Exception('Needs kernel md support')

    def extract(self):
        pct = 0
        speed = 0
        nr = 0
        for l in self.splitlines():
            if len(l) < 2: continue
            if l[1] in ('recovery', 'reshape', 'resync'):
                nr += 1
                pct += int(l[3][0:2].strip('.%'))
                speed += int(l[6].strip('sped=K/sc')) * 1024
        if nr:
            pct = pct / nr
        else:
            pct = 100
        self.val['text'] = '%s %s' % (cprint(pct, 'p', 3, 34), cprint(speed, 'd', 5, 1024))

# vim:ts=4:sw=4:et



### Author: Vikas Gorur (http://github.com/vikasgorur)

class dstat_plugin(dstat):
    """
    Waiting calls on mounted FUSE filesystems

    Displays the number of waiting calls on all mounted FUSE filesystems.
    """

    def __init__(self):
        self.name = 'fuse'
        self.type = 'd'
        self.fusectl_path = "/sys/fs/fuse/connections/"
        self.dirs = []

    def check(self):
        info(1, "Module %s is still experimental." % self.filename)

        if not os.path.exists(self.fusectl_path):
            raise Exception('%s not mounted' % self.fusectl_path)
        if len(os.listdir(self.fusectl_path)) == 0:
            raise Exception('No fuse filesystems mounted')

    def vars(self):
        self.dirs = os.listdir(self.fusectl_path)

        atleast_one_ok = False
        for d in self.dirs:
            if os.access(self.fusectl_path + d + "/waiting", os.R_OK):
                atleast_one_ok = True

        if not atleast_one_ok:
            raise Exception('User is not root or no fuse filesystems mounted')

        return self.dirs

    def extract(self):
        for d in self.dirs:
            path = self.fusectl_path + d + "/waiting"
            if os.path.exists(path):
                line = dopen(path).readline()
                self.val[d] = int(line)
            else:
                self.val[d] = 0

# vim:ts=4:sw=4:et
